@@ -31,7 +31,7 @@
 
 #include <sqlite.h>
 
-static sqlite *db;
+static sqlite *db = NULL; 
 
 extern void migrate_old_categories (sqlite *db);
 
@@ -77,13 +77,18 @@ db_check_tags(void)
       for (iter = list; iter; iter = iter->next)
         {
            gchar *utag = g_ascii_strup (iter->data, -1);
-           r = sqlite_exec_printf (db, "update contacts set tag='%q' where (tag='%q')", 
+           if (strcmp(iter->data, utag))
+             {
+               r = sqlite_exec_printf (db, "update contacts set tag='%q' where (tag='%q')", 
                                    NULL, NULL, &err, utag, iter->data);
-           if (r) 
-             fprintf(stderr, "Err: %s\n", err);
-           if (r)
-             free (err);
+               if (r) 
+                 {
+                   fprintf(stderr, "Err: %s\n", err);
+                   free (err);
+                 }
+             }
            g_free(iter->data);
+           g_free(utag);
         }
       g_slist_free(list);
     }
@@ -98,7 +103,8 @@ db_open (gboolean open_vcard)
 
   buf = g_strdup_printf("%s/%s", g_get_home_dir(), 
                                  open_vcard ? DB_NAME_VCARD : DB_NAME_CONTACTS);
-  
+  if (db) 
+    sqlite_close(db);
   db = sqlite_open (buf, 0, &errmsg);
   g_free (buf);
 	
@@ -228,7 +234,9 @@ new_person_id (guint * id)
 gint 
 sort_entries(struct person * a, struct person * b)
 {
-  return strcoll (a->name, b->name);
+  gint s = strcoll (a->family_name, b->family_name);
+  
+  return s ? s : strcoll (a->name, b->name);
 }
 
 static int
@@ -240,15 +248,6 @@ read_name (void *arg, int argc, char **argv, char **names)
   return 0;
 }
 
-static int
-read_given_name (void *arg, int argc, char **argv, char **names)
-{
-  struct person *p = (struct person *)arg;
-  p->given_name = g_strdup (argv[1]);
-
-  return 0;
-}
-
 static int
 read_family_name (void *arg, int argc, char **argv, char **names)
 {
@@ -258,24 +257,6 @@ read_family_name (void *arg, int argc, char **argv, char **names)
   return 0;
 }
 
-static int
-read_middle_name (void *arg, int argc, char **argv, char **names)
-{
-  struct person *p = (struct person *)arg;
-  p->middle_name = g_strdup (argv[1]);
-
-  return 0;
-}
-static int
-
-read_company (void *arg, int argc, char **argv, char **names)
-{
-  struct person *p = (struct person *)arg;
-  p->company = g_strdup (argv[1]);
-
-  return 0;
-}
-
 static int
 read_one_entry (void *arg, int argc, char **argv, char **names)
 {
@@ -285,15 +266,9 @@ read_one_entry (void *arg, int argc, char **argv, char **names)
   p->id = atoi (argv[0]);
   sqlite_exec_printf (db, "select tag,value from contacts where (urn=%d) and ((tag='NAME') or (tag='name'))",
 		      read_name, p, NULL, p->id);
-  sqlite_exec_printf (db, "select tag,value from contacts where (urn=%d) and ((tag='GIVEN_NAME') or (tag='given_name'))",
-		      read_given_name, p, NULL, p->id);
-  sqlite_exec_printf (db, "select tag,value from contacts where (urn=%d) and (tag='MIDDLE_NAME')",
-		      read_middle_name, p, NULL, p->id);
   sqlite_exec_printf (db, "select tag,value from contacts where (urn=%d) and ((tag='FAMILY_NAME') or (tag='family_name'))",
 		      read_family_name, p, NULL, p->id);
-  sqlite_exec_printf (db, "select tag,value from contacts where (urn=%d) and (tag='COMPANY')",
-		      read_company, p, NULL, p->id);
-
+  
   /* we support contacts without name now */
   if (!p->name) 
     p->name = g_strdup("");
@@ -488,10 +463,10 @@ db_find_tag (struct person *p, gchar * tag)
     {
       struct tag_value *id = (struct tag_value *) iter->data;
       if (!strcasecmp (id->tag, tag))
-	{
-	  t = id;
-	  break;
-	}
+       {
+         t = id;
+         break;
+       }
     }
   return t;
 }
