@@ -8,7 +8,7 @@
  */
 
 /*
- * $Id: main.c,v 1.103 2004/10/26 11:49:05 florian Exp $
+ * $Id: main.c,v 1.109 2005/02/16 14:17:54 florian Exp $
  *
  */
 
@@ -394,7 +394,7 @@ build_children (GtkWidget *vbox, GSList *children, struct person *p)
                 l = gtk_label_new (e->name);
                 gtk_misc_set_alignment(GTK_MISC(l), 0.0, 0.5);
                 memset(&tm, 0, sizeof(tm));
-                sscanf(tv->value,"%04d%02d%02d",&year,&month,&day);
+                sscanf(tv->value,"%04d%02d%02d", &year, &month, &day);
                 if (year)
                   { 
                     tm.tm_year = year - 1900;
@@ -403,7 +403,7 @@ build_children (GtkWidget *vbox, GSList *children, struct person *p)
                     strftime(buf, sizeof(buf), "%x", &tm);
                   }
                 else 
-                  snprintf(buf,sizeof(buf),"%02d/%02d",month+1,day);
+                  snprintf(buf,sizeof(buf),"%02d/%02d", month+1, day);
                 w = gtk_label_new(buf);
                 gtk_misc_set_alignment(GTK_MISC(w), 0.0, 0.5);
                 hbox = gtk_hbox_new(FALSE, gpe_get_boxspacing());
@@ -413,6 +413,35 @@ build_children (GtkWidget *vbox, GSList *children, struct person *p)
               }
           }
           break;
+          case ITEM_IMAGE:
+          {
+            GtkWidget *l = gtk_label_new (e->name);
+            GtkWidget *hbox, *image, *sw, *vp;
+            
+            if (!(tv && tv->value))
+              continue;
+            result |= pop_singles (vbox, singles, p);
+            singles = NULL;
+            hbox = gtk_hbox_new (FALSE, gpe_get_boxspacing());
+            gtk_misc_set_alignment (GTK_MISC (l), 0.0, 0.0);
+            gtk_box_pack_start (GTK_BOX (hbox), l, FALSE, TRUE, 0);
+  
+            image = gtk_image_new();
+            gtk_image_set_from_file(GTK_IMAGE(image), tv->value);
+            /* todo: image size check */
+/*            vp = gtk_viewport_new(NULL, NULL);
+            gtk_viewport_set_shadow_type(GTK_VIEWPORT(vp), GTK_SHADOW_NONE);
+            sw = gtk_scrolled_window_new(NULL, NULL);
+            gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw), 
+                                           GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
+            gtk_container_add(GTK_CONTAINER(vp), image);
+            gtk_container_add(GTK_CONTAINER(sw), vp);
+            gtk_box_pack_start (GTK_BOX (hbox), sw, TRUE, TRUE, 0);
+*/ 
+            gtk_box_pack_start (GTK_BOX (hbox), image, TRUE, TRUE, 0);
+            gtk_widget_show_all(hbox);
+            gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, TRUE, 0);
+          }
           default:
             /* just ignore */
           break;
@@ -457,8 +486,10 @@ show_details (struct person *p)
           GtkWidget *vbox = gtk_vbox_new(FALSE, gpe_get_boxspacing());
           gchar *catstring = build_categories_string(p);
         
-          gtk_table_attach(GTK_TABLE(table), vbox, 0, 1, 0, 1, GTK_FILL, GTK_FILL, 0, 0);
-          for (page = edit_pages; page; page = page->next) /* we use data layout to structure widgets */
+          gtk_table_attach(GTK_TABLE(table), vbox, 0, 1, 0, 1, 
+                           GTK_FILL | GTK_EXPAND, GTK_FILL | GTK_EXPAND, 0, 0);
+          /* we use data layout to structure widgets */
+          for (page = edit_pages; page; page = page->next) 
             {
                edit_thing_t e = page->data;
                /* add section content */
@@ -738,7 +769,8 @@ do_search (GObject *obj, GtkWidget *entry)
 {
   gchar *text = g_utf8_strdown (gtk_entry_get_text (GTK_ENTRY (entry)), -1);
   guint category = gtk_option_menu_get_history (GTK_OPTION_MENU (categories_smenu));
-  GSList *all_entries = db_get_entries (), *iter = NULL;
+  gchar *cat_id = NULL;
+  GSList *sel_entries = NULL, *iter = NULL;
   struct gpe_pim_category *c = NULL;
   GtkTreePath *path;
 
@@ -747,33 +779,35 @@ do_search (GObject *obj, GtkWidget *entry)
       GSList *l = gpe_pim_categories_list ();
       GSList *ll = g_slist_nth (l, category - 1);
 
-      if (ll)
-        c = ll->data;
+      if (ll) 
+        {
+          c = ll->data;
+          cat_id = g_strdup_printf("%u", c->id);
+        }
 
       g_slist_free (l);
     }
 
-  all_entries = g_slist_sort (all_entries, (GCompareFunc)sort_entries);
+  /* Todo cat  */
+  sel_entries = db_get_entries_list (text, cat_id);
+  sel_entries = g_slist_sort (sel_entries, (GCompareFunc)sort_entries);
 
   gtk_list_store_clear (list_store);
   show_details(NULL);
 
-  for (iter = all_entries; iter; iter = iter->next)
+  for (iter = sel_entries; iter; iter = iter->next)
     {
       struct person *p = iter->data;
       GtkTreeIter titer;
       
-      if (match_for_search (p, text, c))
-        {
-	      gtk_list_store_append (list_store, &titer);
-	      gtk_list_store_set (list_store, &titer, 0, p->name, 1, p->id, -1);
-        }
+      gtk_list_store_append (list_store, &titer);
+      gtk_list_store_set (list_store, &titer, 0, p->name, 1, p->id, -1);
 
       discard_person (p);
     }
   
-  g_slist_free (all_entries);
-  if (text) 
+  g_slist_free (sel_entries);
+  if (text)
     g_free (text);
   
   /* select first */
@@ -786,7 +820,8 @@ static void
 do_find (void)
 {
   guint category = gtk_option_menu_get_history (GTK_OPTION_MENU (categories_smenu));
-  GSList *all_entries = do_find_contacts(GTK_WINDOW(mainw)), *iter = NULL;
+  gchar *cat_id = NULL;
+  GSList *all_entries = NULL, *iter = NULL;
   struct gpe_pim_category *c = NULL;
   GtkTreePath *path;
 
@@ -796,11 +831,15 @@ do_find (void)
       GSList *ll = g_slist_nth (l, category - 1);
 
       if (ll)
-        c = ll->data;
+        {
+          c = ll->data;
+          cat_id = g_strdup_printf("%u", c->id);
+        }
 
       g_slist_free (l);
     }
 
+  all_entries = do_find_contacts(GTK_WINDOW(mainw), cat_id);
   all_entries = g_slist_sort (all_entries, (GCompareFunc)sort_entries);
 
   gtk_list_store_clear (list_store);
@@ -1097,7 +1136,10 @@ list_button_press_event (GtkWidget *widget, GdkEventButton *b, GtkListStore *lis
             }
           else
             {
-                edit_contact(widget, NULL);
+                if (mode_landscape)
+                  edit_contact(widget, NULL);
+                else
+                  show_details_window(widget);
             }
         }
     }
@@ -1150,6 +1192,70 @@ on_main_focus(GtkWindow *window,  GdkEventExpose *event,gpointer user_data)
   gtk_editable_select_region(GTK_EDITABLE(searchentry),0,-1);
 }
 
+static int 
+import_one_file(const gchar *filename)
+{
+  GError *err = NULL;
+  gchar *content = NULL;
+  gsize count = 0;
+  int result = 0;
+	
+  if (g_file_get_contents(filename, &content, &count, &err))
+    {
+      result = import_vcard(content, count);
+      g_free(content);
+    }
+  else
+    result = -1;
+  return result;
+}
+
+static void
+on_import_vcard (GtkWidget *widget, gpointer data)
+{
+  GtkWidget *filesel, *feedbackdlg;
+  
+  filesel = gtk_file_selection_new(_("Choose file"));
+  gtk_file_selection_set_select_multiple(GTK_FILE_SELECTION(filesel), TRUE);
+  
+  if (gtk_dialog_run(GTK_DIALOG(filesel)) == GTK_RESPONSE_OK)
+    {
+      gchar *errstr = NULL;
+      int ec = 0, i = 0;
+      gchar **files = 
+        gtk_file_selection_get_selections(GTK_FILE_SELECTION(filesel));
+      gtk_widget_hide(filesel); 
+      while (files[i])
+        {
+          if (import_one_file(files[i]) < 0) 
+            {
+              gchar *tmp;
+              if (!errstr) 
+                errstr=g_strdup("");
+              ec++;
+			  tmp = g_strdup_printf("%s\n%s",errstr,strrchr(files[i],'/')+1);
+              if (errstr) 
+                 g_free(errstr);
+              errstr = tmp;
+            }
+          i++;  
+        }
+      if (ec)
+        feedbackdlg = gtk_message_dialog_new(GTK_WINDOW(mainw),
+          GTK_DIALOG_MODAL, GTK_MESSAGE_INFO, GTK_BUTTONS_OK, 
+          "%s %i %s\n%s",_("Import of"),ec,_("files failed:"),errstr);
+      else
+        feedbackdlg = gtk_message_dialog_new(GTK_WINDOW(mainw),
+          GTK_DIALOG_MODAL, GTK_MESSAGE_INFO, GTK_BUTTONS_OK, 
+          _("Import successful"));
+      gtk_dialog_run(GTK_DIALOG(feedbackdlg));
+      gtk_widget_destroy(feedbackdlg);
+    }
+  gtk_widget_destroy(filesel);
+  db_open(FALSE);
+  update_display();
+}
+
 static GtkWidget *
 create_main (gboolean edit_structure)
 {
@@ -1233,6 +1339,12 @@ create_main (gboolean edit_structure)
 			    _("Properties"), _("Tap here to configure this program."),
   			    G_CALLBACK (configure), (gpointer)edit_structure, -1);
 				
+  pw = gtk_image_new_from_stock(GTK_STOCK_OPEN, 
+                                gtk_toolbar_get_icon_size(GTK_TOOLBAR (toolbar)));
+  gtk_toolbar_append_item (GTK_TOOLBAR (toolbar),
+			    _("Import"), _("Open file to import a contact from it."), 
+                NULL, pw, G_CALLBACK (on_import_vcard), NULL);
+    
   gtk_toolbar_append_space(GTK_TOOLBAR(toolbar));
 
   gtk_toolbar_insert_stock (GTK_TOOLBAR (toolbar), GTK_STOCK_QUIT,
@@ -1348,8 +1460,8 @@ create_main (gboolean edit_structure)
       gtk_box_pack_start (GTK_BOX (vbox1), pDetail, FALSE, TRUE, 0);
     }
   tabDetail = gtk_table_new (1, 2, FALSE);
-  gtk_table_set_col_spacings(GTK_TABLE(tabDetail),gpe_get_boxspacing()*2);
-  gtk_container_set_border_width(GTK_CONTAINER(tabDetail),gpe_get_border());
+  gtk_table_set_col_spacings(GTK_TABLE(tabDetail), gpe_get_boxspacing() * 2);
+  gtk_container_set_border_width(GTK_CONTAINER(tabDetail), gpe_get_border());
   if (mode_large_screen)
     {
       GtkWidget *vport;
@@ -1370,36 +1482,14 @@ create_main (gboolean edit_structure)
 
   g_signal_connect (G_OBJECT (tree_sel), "changed",
 		    G_CALLBACK (selection_made), main_window);
- 
   g_signal_connect (G_OBJECT (main_window), "focus-in-event",
 		    G_CALLBACK (on_main_focus), entry1);
 
   displaymigration_mark_window (main_window);
-
   
   return main_window;
 }
 
-
-static int 
-import_one_file(const gchar *filename)
-{
-  GError *err = NULL;
-  gchar *content = NULL;
-  gsize count = 0;
-  int result = 0;
-	
-  if (g_file_get_contents(filename, &content, &count, &err))
-    {
-      result = import_vcard(content, count);
-      g_free(content);
-    }
-  else
-    result = -1;
-  return result;
-}
-
-
 int
 main (int argc, char *argv[])
 {
@@ -1448,14 +1538,11 @@ main (int argc, char *argv[])
   if (ifile)
     {
       int ret;
-      /*GtkWidget *dialog = NULL;*/
+      GtkWidget *dialog = NULL;
       
       ret =  import_one_file(ifile);
       if (ret)
-        printf("Could not import file %s.\n", ifile);
-      else
-        printf("Successfully imported %s.\n", ifile);
-/*       dialog = gtk_message_dialog_new(NULL,0,GTK_MESSAGE_INFO,
+       dialog = gtk_message_dialog_new(NULL,0,GTK_MESSAGE_INFO,
 	                                    GTK_BUTTONS_OK,
 	                                    _("Could not import file %s."),
 	                                     ifile);
@@ -1466,7 +1553,7 @@ main (int argc, char *argv[])
 	                                      ifile);
       gtk_dialog_run(GTK_DIALOG(dialog));
       gtk_widget_destroy(dialog);
-*/      
+      
       exit (EXIT_SUCCESS);
     }
   
